@@ -37,7 +37,7 @@ _UNSUPPORTED_FUNCTIONS = {
     np.busday_offset,
     np.is_busday,
     # not clear how to approach
-    np.piecewise,  # astropy.units doens't have a simple implementation either
+    np.piecewise,  # astropy.units doesn't have a simple implementation either
     np.packbits,
     np.unpackbits,
     np.ix_,
@@ -158,38 +158,169 @@ def _sanitize_range(_range, units):
     return new_range.squeeze()
 
 
-@implements(np.histogram)
-def histogram(
-    a,
-    bins=10,
-    range=None,
-    *args,
-    **kwargs,
+def _histogram(a, *, bins=10, range=None, density=None, weights=None, normed=None):
+    range = _sanitize_range(range, units=[getattr(a, "units", None)])
+    if NUMPY_VERSION >= Version("1.24"):
+        counts, bins = np.histogram._implementation(
+            np.asarray(a),
+            bins=bins,
+            range=range,
+            density=density,
+            weights=np.asarray(weights) if weights is not None else None,
+        )
+    else:
+        counts, bins = np.histogram._implementation(
+            np.asarray(a),
+            bins=bins,
+            range=range,
+            normed=normed,
+            weights=np.asarray(weights) if weights is not None else None,
+            density=density,
+        )
+    # a and/or weights could have units, only apply if present
+    # don't getattr(..., "units", NULL_UNIT) because e.g. we don't want
+    # a unyt_array if weights are not a unyt_array and not density
+    if density and hasattr(a, "units"):
+        counts /= a.units
+    if weights is not None and hasattr(weights, "units"):
+        counts *= weights.units
+    return counts, bins * getattr(a, "units", 1)
+
+
+if NUMPY_VERSION >= Version("1.24"):
+
+    @implements(np.histogram)
+    def histogram(a, bins=10, range=None, density=None, weights=None):
+        return _histogram(a, bins=bins, range=range, density=density, weights=weights)
+
+else:
+
+    @implements(np.histogram)
+    def histogram(a, bins=10, range=None, normed=None, weights=None, density=None):
+        return _histogram(
+            a, bins=bins, range=range, normed=normed, weights=weights, density=density
+        )
+
+
+def _histogram2d(x, y, *, bins=10, range=None, density=None, weights=None, normed=None):
+    range = _sanitize_range(
+        range, units=[getattr(x, "units", None), getattr(y, "units", None)]
+    )
+    if NUMPY_VERSION >= Version("1.24"):
+        counts, xbins, ybins = np.histogram2d._implementation(
+            np.asarray(x),
+            np.asarray(y),
+            bins=bins,
+            range=range,
+            density=density,
+            weights=np.asarray(weights) if weights is not None else None,
+        )
+    else:
+        counts, xbins, ybins = np.histogram2d._implementation(
+            np.asarray(x),
+            np.asarray(y),
+            bins=bins,
+            range=range,
+            normed=normed,
+            weights=np.asarray(weights) if weights is not None else None,
+            density=density,
+        )
+    # x, y and/or weights could have units, only apply if present
+    # don't getattr(..., "units", NULL_UNIT) because e.g. we don't want
+    # a unyt_array if weights are not a unyt_array and not density
+    if density:
+        if hasattr(x, "units"):
+            counts /= x.units
+        if hasattr(y, "units"):
+            counts /= y.units
+    if weights is not None and hasattr(weights, "units"):
+        counts *= weights.units
+    return counts, xbins * getattr(x, "units", 1), ybins * getattr(y, "units", 1)
+
+
+if NUMPY_VERSION >= Version("1.24"):
+
+    @implements(np.histogram2d)
+    def histogram2d(x, y, bins=10, range=None, density=None, weights=None):
+        return _histogram2d(
+            x, y, bins=bins, range=range, density=density, weights=weights
+        )
+
+else:
+
+    @implements(np.histogram2d)
+    def histogram2d(x, y, bins=10, range=None, normed=None, weights=None, density=None):
+        return _histogram2d(
+            x,
+            y,
+            bins=bins,
+            range=range,
+            normed=normed,
+            weights=weights,
+            density=density,
+        )
+
+
+def _histogramdd(
+    sample, *, bins=10, range=None, density=None, weights=None, normed=None
 ):
-    range = _sanitize_range(range, units=[a.units])
-    counts, bins = np.histogram._implementation(
-        np.asarray(a), bins, range, *args, **kwargs
-    )
-    return counts, bins * a.units
+    range = _sanitize_range(range, units=[getattr(_, "units", None) for _ in sample])
+    if NUMPY_VERSION >= Version("1.24"):
+        counts, bins = np.histogramdd._implementation(
+            [np.asarray(_) for _ in sample],
+            bins=bins,
+            range=range,
+            density=density,
+            weights=np.asarray(weights) if weights is not None else None,
+        )
+    else:
+        counts, bins = np.histogramdd._implementation(
+            [np.asarray(_) for _ in sample],
+            bins=bins,
+            range=range,
+            normed=normed,
+            weights=np.asarray(weights) if weights is not None else None,
+            density=density,
+        )
+    # sample(s) and/or weights could have units, only apply if present
+    # don't getattr(..., "units", NULL_UNIT) because e.g. we don't want
+    # a unyt_array if weights are not a unyt_array and not density
+    if density:
+        divider_units = 1 * NULL_UNIT
+        for s in sample:
+            if not hasattr(s, "units"):
+                continue
+            divider_units *= s.units
+        if divider_units != NULL_UNIT:
+            counts /= divider_units
+
+    if weights is not None and hasattr(weights, "units"):
+        counts *= weights.units
+    return counts, tuple(_bin * getattr(s, "units", 1) for _bin, s in zip(bins, sample))
 
 
-@implements(np.histogram2d)
-def histogram2d(x, y, bins=10, range=None, *args, **kwargs):
-    range = _sanitize_range(range, units=[x.units, y.units])
-    counts, xbins, ybins = np.histogram2d._implementation(
-        np.asarray(x), np.asarray(y), bins, range, *args, **kwargs
-    )
-    return counts, xbins * x.units, ybins * y.units
+if NUMPY_VERSION >= Version("1.24"):
 
+    @implements(np.histogramdd)
+    def histogramdd(sample, bins=10, range=None, density=None, weights=None):
+        return _histogramdd(
+            sample, bins=bins, range=range, density=density, weights=weights
+        )
 
-@implements(np.histogramdd)
-def histogramdd(sample, bins=10, range=None, *args, **kwargs):
-    units = [_.units for _ in sample]
-    range = _sanitize_range(range, units=units)
-    counts, bins = np.histogramdd._implementation(
-        [np.asarray(_) for _ in sample], bins, range, *args, **kwargs
-    )
-    return counts, tuple(_bin * u for _bin, u in zip(bins, units))
+else:
+
+    @implements(np.histogramdd)
+    def histogramdd(
+        sample, bins=10, range=None, normed=None, weights=None, density=None
+    ):
+        return _histogramdd(
+            sample,
+            bins=bins,
+            range=range,
+            normed=normed,
+            weights=weights,
+            density=density,
+        )
 
 
 @implements(np.histogram_bin_edges)
@@ -357,73 +488,73 @@ def block(arrays):
 
 
 @implements(np.fft.fft)
-def ftt_fft(a, *args, **kwargs):
-    return np.fft.fft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_fft(a, *args, **kwargs):
+    return np.fft.fft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.fft2)
-def ftt_fft2(a, *args, **kwargs):
-    return np.fft.fft2._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_fft2(a, *args, **kwargs):
+    return np.fft.fft2._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.fftn)
-def ftt_fftn(a, *args, **kwargs):
-    return np.fft.fftn._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_fftn(a, *args, **kwargs):
+    return np.fft.fftn._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.hfft)
-def ftt_hfft(a, *args, **kwargs):
-    return np.fft.hfft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_hfft(a, *args, **kwargs):
+    return np.fft.hfft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.rfft)
-def ftt_rfft(a, *args, **kwargs):
-    return np.fft.rfft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_rfft(a, *args, **kwargs):
+    return np.fft.rfft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.rfft2)
-def ftt_rfft2(a, *args, **kwargs):
-    return np.fft.rfft2._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_rfft2(a, *args, **kwargs):
+    return np.fft.rfft2._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.rfftn)
-def ftt_rfftn(a, *args, **kwargs):
-    return np.fft.rfftn._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_rfftn(a, *args, **kwargs):
+    return np.fft.rfftn._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.ifft)
-def ftt_ifft(a, *args, **kwargs):
-    return np.fft.ifft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_ifft(a, *args, **kwargs):
+    return np.fft.ifft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.ifft2)
-def ftt_ifft2(a, *args, **kwargs):
-    return np.fft.ifft2._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_ifft2(a, *args, **kwargs):
+    return np.fft.ifft2._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.ifftn)
-def ftt_ifftn(a, *args, **kwargs):
-    return np.fft.ifftn._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_ifftn(a, *args, **kwargs):
+    return np.fft.ifftn._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.ihfft)
-def ftt_ihfft(a, *args, **kwargs):
-    return np.fft.ihfft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_ihfft(a, *args, **kwargs):
+    return np.fft.ihfft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.irfft)
-def ftt_irfft(a, *args, **kwargs):
-    return np.fft.irfft._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_irfft(a, *args, **kwargs):
+    return np.fft.irfft._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.irfft2)
-def ftt_irfft2(a, *args, **kwargs):
-    return np.fft.irfft2._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_irfft2(a, *args, **kwargs):
+    return np.fft.irfft2._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.irfftn)
-def ftt_irfftn(a, *args, **kwargs):
-    return np.fft.irfftn._implementation(np.asarray(a), *args, **kwargs) / a.units
+def fft_irfftn(a, *args, **kwargs):
+    return np.fft.irfftn._implementation(np.asarray(a), *args, **kwargs) * a.units
 
 
 @implements(np.fft.fftshift)
@@ -490,26 +621,94 @@ def array_equiv(a1, a2, *args, **kwargs) -> bool:
     )
 
 
-@implements(np.linspace)
-def linspace(start, stop, *args, **kwargs):
+def _linspace(
+    start,
+    stop,
+    num=50,
+    endpoint=True,
+    retstep=False,
+    dtype=None,
+    axis=0,
+    *,
+    device=None,
+):
     _validate_units_consistency((start, stop))
-    return (
-        np.linspace._implementation(
-            np.asarray(start), np.asarray(stop), *args, **kwargs
+    kwargs = {
+        "num": num,
+        "endpoint": endpoint,
+        "retstep": retstep,
+        "dtype": dtype,
+        "axis": axis,
+    }
+    if NUMPY_VERSION >= Version("2.1.0.dev0"):
+        kwargs["device"] = device
+    result = np.linspace._implementation(np.asarray(start), np.asarray(stop), **kwargs)
+    if retstep:
+        return result[0] * start.units, result[1] * start.units
+    else:
+        return result * start.units
+
+
+if NUMPY_VERSION >= Version("2.0.0.dev0"):
+
+    @implements(np.linspace)
+    def linspace(
+        start,
+        stop,
+        num=50,
+        endpoint=True,
+        retstep=False,
+        dtype=None,
+        axis=0,
+        *,
+        device=None,
+    ):
+        return _linspace(
+            start,
+            stop,
+            num=num,
+            endpoint=endpoint,
+            retstep=retstep,
+            dtype=dtype,
+            axis=axis,
+            device=device,
+        )
+
+else:
+
+    @implements(np.linspace)
+    def linspace(start, stop, num=50, endpoint=True, retstep=False, dtype=None, axis=0):
+        return _linspace(
+            start,
+            stop,
+            num=num,
+            endpoint=endpoint,
+            retstep=retstep,
+            dtype=dtype,
+            axis=axis,
         )
-        * start.units
-    )
 
 
 @implements(np.logspace)
-def logspace(start, stop, *args, **kwargs):
-    _validate_units_consistency((start, stop))
-    return (
-        np.logspace._implementation(
-            np.asarray(start), np.asarray(stop), *args, **kwargs
+def logspace(start, stop, num=50, endpoint=True, base=10, dtype=None, axis=0):
+    startu = getattr(start, "units", NULL_UNIT)
+    stopu = getattr(stop, "units", NULL_UNIT)
+    if startu != NULL_UNIT or stopu != NULL_UNIT:
+        raise TypeError(
+            "The first two arguments to numpy.logspace must be dimensionless, "
+            f"got units={startu} (arg1) and units={stopu} (arg2). If output with"
+            " units is desired, apply the units to the `base` kwarg."
         )
-        * start.units
+    result = np.logspace._implementation(
+        np.asarray(start),
+        np.asarray(stop),
+        num=num,
+        endpoint=endpoint,
+        base=np.asarray(base),
+        dtype=dtype,
+        axis=axis,
     )
+    return result * getattr(base, "units", NULL_UNIT)
 
 
 @implements(np.geomspace)
@@ -535,7 +734,8 @@ def copyto(dst, src, *args, **kwargs):
 
 @implements(np.prod)
 def prod(a, *args, **kwargs):
-    return np.prod._implementation(np.asarray(a), *args, **kwargs) * a.units**a.size
+    res = np.prod._implementation(np.asarray(a), *args, **kwargs)
+    return res * a.units ** (a.size // res.size)
 
 
 @implements(np.var)
@@ -570,8 +770,9 @@ def nanquantile(a, *args, **kwargs):
 
 @implements(np.linalg.det)
 def linalg_det(a, *args, **kwargs):
-    return np.linalg.det._implementation(np.asarray(a), *args, **kwargs) * a.units ** (
-        a.shape[0]
+    return (
+        np.linalg.det._implementation(np.asarray(a), *args, **kwargs)
+        * a.units ** (a.shape[0])
     )
 
 
@@ -649,7 +850,7 @@ def savetxt(fname, X, *args, **kwargs):
 
 @implements(np.apply_over_axes)
 def apply_over_axes(func, a, axes):
-    res = func(np.asarray(a), axes[0]) * a.units
+    res = func(a, axes[0])
     if len(axes) > 1:
         # this function is recursive by nature,
         # here we intentionally do not call the base _implementation
@@ -696,6 +897,13 @@ def cumprod(a, *args, **kwargs):
     )
 
 
+if NUMPY_VERSION >= Version("2.1.0.dev0"):
+
+    @implements(np.cumulative_prod)
+    def cumulative_prod(x, /, *args, **kwargs):
+        return cumprod(x, *args, **kwargs)
+
+
 @implements(np.pad)
 def pad(array, *args, **kwargs):
     return np.pad._implementation(np.asarray(array), *args, **kwargs) * array.units
@@ -816,8 +1024,7 @@ def sinc(x, *args, **kwargs):
     return np.sinc._implementation(np.asarray(x), *args, **kwargs)
 
 
-@implements(np.clip)
-def clip(a, a_min, a_max, out=None, *args, **kwargs):
+def clip_impl(a, a_min, a_max, out=None, *args, **kwargs):
     _validate_units_consistency_v2(a.units, a_min, a_max)
     if out is None:
         return (
@@ -843,6 +1050,20 @@ def clip(a, a_min, a_max, out=None, *args, **kwargs):
     return unyt_array(res, a.units, bypass_validation=True)
 
 
+if NUMPY_VERSION >= Version("2.1.0.dev0"):
+    from numpy._globals import _NoValue
+
+    @implements(np.clip)
+    def clip(a, a_min=_NoValue, a_max=_NoValue, out=None, *args, **kwargs):
+        return clip_impl(a, a_min, a_max, out, *args, **kwargs)
+
+else:
+
+    @implements(np.clip)
+    def clip(a, a_min, a_max, out=None, *args, **kwargs):
+        return clip_impl(a, a_min, a_max, out, *args, **kwargs)
+
+
 @implements(np.where)
 def where(condition, *args, **kwargs):
     if len(args) == 0:
@@ -999,3 +1220,23 @@ if hasattr(np, "in1d"):
         return np.isin._implementation(
             np.asarray(ar1), np.asarray(ar2), *args, **kwargs
         )
+
+
+@implements(np.take)
+def take(a, indices, axis=None, out=None, mode="raise"):
+    ret_units = getattr(a, "units", NULL_UNIT)
+
+    if out is not None:
+        out_view = np.asarray(out)
+    else:
+        out_view = None
+
+    res = np.take._implementation(
+        np.asarray(a), indices, axis=axis, out=out_view, mode=mode
+    )
+
+    if getattr(out, "units", None) is not None:
+        out.units = ret_units
+
+    ret_cls = unyt_quantity if res.ndim == 0 else unyt_array
+    return ret_cls(res, ret_units, bypass_validation=True)
