@@ -132,6 +132,10 @@ from unyt.unit_symbols import delta_degC, delta_degF
 
 from ._deprecation import warn_deprecated
 
+NUMPY_VERSION = Version(version("numpy"))
+if NUMPY_VERSION >= Version("2.0.0.dev0"):
+    from numpy import vecdot
+
 NULL_UNIT = Unit()
 POWER_MAPPING = {multiply: lambda x: x, divide: lambda x: 2 - x}
 DISALLOWED_DTYPES = (
@@ -424,6 +428,8 @@ binary_operators = (
     divmod_,
     heaviside,
 )
+if NUMPY_VERSION >= Version("2.0.0.dev0"):
+    binary_operators += (vecdot,)
 
 trigonometric_operators = (sin, cos, tan)
 
@@ -576,6 +582,8 @@ class unyt_array(np.ndarray):
         matmul: _multiply_units,
         clip: _passthrough_unit,
     }
+    if NUMPY_VERSION >= Version("2.0.0.dev0"):
+        _ufunc_registry[vecdot] = _multiply_units
 
     __array_priority__ = 2.0
 
@@ -950,7 +958,7 @@ class unyt_array(np.ndarray):
             quantity, try the :meth:`list_equivalencies`
             method. Default: None
         kwargs: optional
-            Any additional keywoard arguments are supplied to the
+            Any additional keyword arguments are supplied to the
             equivalence
 
         Raises
@@ -1554,7 +1562,7 @@ class unyt_array(np.ndarray):
         else:
             g = f
         dataset = g[dataset_name]
-        data = dataset[:]
+        data = dataset[...]
         units = dataset.attrs.get("units", "")
         unit_lut = default_unit_symbol_lut.copy()
         unit_lut_load = pickle.loads(dataset.attrs["unit_registry"].tobytes())
@@ -1764,8 +1772,8 @@ class unyt_array(np.ndarray):
         Power function
         """
         # see https://github.com/yt-project/unyt/issues/203
-        if p == 0.0:
-            ret = self.ua
+        if np.isscalar(p) and p == 0.0:
+            ret = self.unit_array
             ret.units = Unit("dimensionless")
             return ret
         else:
@@ -1854,17 +1862,32 @@ class unyt_array(np.ndarray):
                 u1 = Unit(registry=getattr(u0, "registry", None))
             elif ufunc is power:
                 u1 = inp1
-                if inp0.shape != () and inp1.shape != ():
-                    raise UnitOperationError(ufunc, u0, u1)
-                if isinstance(u1, unyt_array):
-                    if u1.units.is_dimensionless:
-                        pass
-                    else:
+                if inp0.shape == () or inp1.shape == ():
+                    if isinstance(u1, unyt_array) and not u1.units.is_dimensionless:
                         raise UnitOperationError(ufunc, u0, u1.units)
-                if u1.shape == ():
-                    u1 = float(u1)
+                    if u1.shape == ():
+                        u1 = float(u1)
+                    else:
+                        u1 = 1.0
+                elif inp0.shape == inp1.shape:
+                    if isinstance(u1, unyt_array) and not u1.units.is_dimensionless:
+                        raise UnitOperationError(ufunc, u0, getattr(u1, "units", None))
+
+                    if (
+                        (isinstance(u0, Unit) and not u0.is_dimensionless)
+                        or isinstance(u0, unyt_array)
+                        and not u0.units.is_dimensionless
+                    ):
+                        # u0 has units
+                        if np.ptp(u1) != 0:
+                            raise UnitOperationError(
+                                ufunc, u0, getattr(u1, "units", None)
+                            )
+
+                    first_element_slice = (0,) * u1.ndim
+                    u1 = float(u1[first_element_slice])
                 else:
-                    u1 = 1.0
+                    raise UnitOperationError(ufunc, u0, u1)
             unit_operator = self._ufunc_registry[ufunc]
 
             if (
@@ -1985,8 +2008,8 @@ class unyt_array(np.ndarray):
                 mul = 1
             else:
                 raise RuntimeError(
-                    "Support for the %s ufunc with %i inputs has not been "
-                    "added to unyt_array." % (str(ufunc), len(inputs))
+                    f"Support for the {ufunc} ufunc with {len(inputs)} "
+                    "inputs has not been added to unyt_array."
                 )
         if unit is None:
             out_arr = np.array(out_arr, copy=_COPY_IF_NEEDED)
@@ -1997,7 +2020,7 @@ class unyt_array(np.ndarray):
         elif out_arr.size == 1:
             out_arr = unyt_array(np.asarray(out_arr), unit)
         else:
-            if ret_class is unyt_quantity:
+            if issubclass(ret_class, unyt_quantity):
                 # This happens if you do ndarray * unyt_quantity.
                 # Explicitly casting to unyt_array avoids creating a
                 # unyt_quantity with size > 1
@@ -2125,6 +2148,21 @@ class unyt_array(np.ndarray):
             out.units = res_units
         return ret
 
+    def take(self, indices, axis=None, out=None, mode="raise"):
+        """method
+
+        Return an array formed from the elements of `a` at the given indices.
+
+        Refer to :func:`numpy.take` for full documentation.
+
+        See also
+        --------
+        numpy.take : equivalent function
+        """
+        from ._array_functions import take
+
+        return take(self, indices, axis=axis, out=out, mode=mode)
+
     def __reduce__(self):
         """Pickle reduction method
 
