@@ -6,7 +6,6 @@ Test unit lookup tables and registry
 
 """
 
-
 import os
 
 import pytest
@@ -45,6 +44,45 @@ def test_add_modify_error():
     assert ureg["tayne"][:3] == ureg["m"][:3]
 
 
+def test_modify_symbol_from_default_unit_registry():
+    # see https://github.com/yt-project/unyt/issues/473
+    from unyt import km
+    from unyt.unit_object import default_unit_registry
+
+    with pytest.raises(TypeError):
+        default_unit_registry.modify("cm", 10 * km)
+
+    with pytest.raises(TypeError):
+        default_unit_registry.remove("cm")
+
+
+def test_modify_cache_clear():
+    # see https://github.com/yt-project/unyt/issues/473
+
+    ureg = UnitRegistry()
+    ureg.add("celery", 1.0, length)
+    u0 = Unit("m", registry=ureg)
+    u1 = Unit("celery", registry=ureg)
+    assert u1 == u0
+
+    ureg.modify("celery", 0.5)
+
+    # check that this equality still holds post registry modification
+    assert u1 == u0
+
+    u2 = Unit("celery", registry=ureg)
+    assert u2 != u1
+    assert 1.0 * u2 == 0.5 * u0
+
+
+def test_remove_unit():
+    ureg = UnitRegistry()
+    ureg.add("celery", 1.0, length)
+    ureg.remove("celery")
+    with pytest.raises(UnitParseError):
+        Unit("celery", registry=ureg)
+
+
 def test_keys():
     ureg = UnitRegistry()
     assert sorted(ureg.keys()) == sorted(ureg.lut.keys())
