@@ -528,6 +528,44 @@ def test_power():
     assert_array_equal_units(cm_arr**0, unyt_quantity(1.0, "dimensionless"))
 
 
+@pytest.mark.parametrize(
+    "p",
+    [
+        pytest.param(unyt_array(0), id="scalar power 0"),
+        pytest.param(unyt_array(1), id="scalar power 1"),
+        pytest.param(unyt_array([0, 0, 0, 0]), id="uniform power 0"),
+        pytest.param(unyt_array([1, 1, 1, 1]), id="uniform power 1"),
+        pytest.param(unyt_array([0, 1, 2, 3]), id="non-uniform power"),
+    ],
+)
+def test_dimensionless_array_power(p):
+    # see https://github.com/yt-project/unyt/issues/522
+    a = unyt_array([1, 2, 3, 4])
+    expected = a.value**p.value
+    assert_array_equal(expected, (a**p).value)
+
+
+@pytest.mark.parametrize(
+    "a, p",
+    [
+        pytest.param(
+            unyt_array([0, 1, 2, 3], "cm"),
+            unyt_array([0, 1, 2, 3]),
+            id="non-uniform power with arr non dimensionless",
+        ),
+        pytest.param(
+            unyt_array([0, 1, 2, 3]),
+            unyt_array([0, 0, 0, 0], "cm"),
+            id="non dimensionless power",
+        ),
+    ],
+)
+def test_invalid_array_power(a, p):
+    # see https://github.com/yt-project/unyt/issues/522
+    with pytest.raises(UnitOperationError):
+        a**p
+
+
 def test_comparisons():
     """
     Test numpy ufunc comparison operators for unit consistency.
@@ -1587,6 +1625,25 @@ def test_h5_io():
     shutil.rmtree(tmpdir)
 
 
+def test_h5_quantity_roundtrip(tmp_path):
+    # regression test for https://github.com/yt-project/unyt/issues/559
+    pytest.importorskip("h5py")
+
+    q1 = 5 * cm
+    savefile = tmp_path / "savefile.hdf5"
+    q1.write_hdf5(
+        savefile,
+        dataset_name="test_ds",
+        group_name="test_group",
+    )
+    q2 = unyt_quantity.from_hdf5(
+        savefile,
+        dataset_name="test_ds",
+        group_name="test_group",
+    )
+    assert_array_equal_units(q2, q1)
+
+
 def test_equivalencies():
     import unyt as u
 
@@ -1788,7 +1845,7 @@ def test_equivalencies():
     assert T.units == u.m.units / u.s.units
     assert c_s.units == u.km.units / u.s.units
 
-    # tempearture <-> energy
+    # temperature <-> energy
 
     mu = 0.5
     gg = 4.0 / 3.0
